#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "MessageBox.h"


/* Demonstrate a Message Box. */


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance
BOOL RegisterWin95( CONST WNDCLASS* lpwc );

LPCTSTR lpszAppName  = "MyApp";
LPCTSTR lpszTitle    = " 4.2";

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

      // Register the main application window class.
   //............................................
   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc; /* window function */      
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;  /* handle to this instance */            
   wc.hIcon         = LoadIcon( hInstance, lpszAppName ); /* icon style */
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);        /* cursor style */
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName; /* window class name */

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

    hInst = hInstance; 

   // Create the main application window.
   //....................................
   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 

}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm  = LoadIcon(wcex.hInstance, "SMALL");
			
   return RegisterClassEx( &wcex );
}


/* This function is called by Windows 95 and is passed 
   messages from the message queue.
*/
LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{  
   int response;
   switch( uMsg )
   {
      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        MessageBeep( MB_ICONEXCLAMATION );
                        if ( MessageBox( hWnd, "Hello!", NULL, 
                                    MB_YESNO | MB_ICONEXCLAMATION ) == IDYES )
                        {
                           // YES was pressed.. 
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      case WM_RBUTTONDOWN: /* process right button */
				response = MessageBox(hWnd, "Press One:", "Right Button",
                                         MB_ABORTRETRYIGNORE);
				switch(response) {
					case IDABORT:
						MessageBox(hWnd, "", "Abort", MB_OK);
						break;
					case IDRETRY:
						MessageBox(hWnd, "", "Retry", MB_OK);
						break;
					case IDIGNORE:
						MessageBox(hWnd, "", "Ignore", MB_OK);
						break;
				}
				break;
	  case WM_LBUTTONDOWN: /* process left button */
				response = MessageBox(hWnd, "Continue?", "Left Button",
					 MB_ICONHAND | MB_YESNO);
				switch(response) {
					case IDYES:
						MessageBox(hWnd, "Press Button", "Yes", MB_OK);
						break;
					case IDNO:
						MessageBox(hWnd, "Press Button", "No", MB_OK);
					break;
				}
				break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}

LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
