#include <windows.h>
#include <commctrl.h>
#include <string.h>

#include "Tree_Ctl.h"
#if defined (win32)
   #define IS_WIN32 TRUE
#else
   #define IS_WIN32 FALSE
#endif

#define NUM 5

HINSTANCE hInst;        // current instance
LPCTSTR lpszAppName = "Generic";
LPCTSTR lpszTitle =   "Generic Application";
BOOL RegisterWin95(CONST WNDCLASS* lpwc);

void InitTree(void);
void report(HDC hdc, char *s);
HWND hTreeWndCtrl;
HTREEITEM hTreeWnd[NUM];
HTREEITEM hTreeCurrent;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
 {
   MSG msg;
   HWND hWnd;
   WNDCLASS wc;
   RECT WinDim;
   HANDLE hAccel;
	
   wc.style          = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc    = (WNDPROC)WndProc;
   wc.cbClsExtra     = 0;
   wc.cbWndExtra     = 0;
   wc.hInstance      = 0;
   wc.hIcon          = LoadIcon(hInstance, lpszAppName);
   wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName   = lpszAppName;
   wc.lpszClassName  = lpszAppName;

   if(!RegisterWin95(&wc))
      return false;
   hInst = hInstance; /* save the current instance handle */
   hWnd = CreateWindow (lpszAppName,
                        lpszTitle,
                        WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT, 0,
                        CW_USEDEFAULT, 0,
                        NULL,
                        NULL,
                        hInstance,
                        NULL
                       );
   if(!hWnd)
      return false;
   InitCommonControls();

   
   GetClientRect(hWnd, &WinDim);  /* get size of parent window */

   /* create a tree view */
   hTreeWndCtrl = CreateWindow(
                  WC_TREEVIEW,
                  "",
                  WS_VISIBLE | WS_TABSTOP | WS_CHILD |
                  TVS_HASLINES | TVS_HASBUTTONS |
                  TVS_LINESATROOT,
                  0, 0, 100, 100,
                  hWnd,
                  NULL,
                  hInst,
                  NULL
             );
   InitTree();

   /* load accelerators */
   hAccel = LoadAccelerators(hInstance, "MYMENU");

   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);
   while(GetMessage(&msg, NULL, 0,0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
   return(msg.wParam);
 }

BOOL RegisterWin95(CONST WNDCLASS* lpwc)
 {
   WNDCLASSEX wcex;

   wcex.style           = lpwc->style;
   wcex.lpfnWndProc     = lpwc->lpfnWndProc;
   wcex.cbClsExtra      = lpwc->cbClsExtra;
   wcex.cbWndExtra      = lpwc->cbWndExtra;
   wcex.hInstance       = lpwc->hInstance;
   wcex.hIcon           = lpwc->hIcon;
   wcex.hCursor         = lpwc->hCursor;
   wcex.hbrBackground   = lpwc->hbrBackground;
   wcex.lpszMenuName    = lpwc->lpszMenuName;
   wcex.lpszClassName   = lpwc->lpszClassName;
   wcex.cbSize          = sizeof(WNDCLASSEX);
   wcex.hIconSm         = LoadIcon(wcex.hInstance, "SMALL");
   return RegisterClassEx(&wcex);
 }

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
  HDC hdc;
  static char selection[80] = "";
  NMHDR *nmptr;
  PAINTSTRUCT paintstruct;
  int i;

   switch(uMsg)
    {
      case WM_COMMAND:
         switch(LOWORD(wParam))
          {
		    case IDM_EXPAND:
               TreeView_Expand(hTreeWndCtrl, hTreeCurrent, TVE_EXPAND);
               break;
            case IDM_EXPANDALL:
               for(i=0; i<NUM; i++)
                  TreeView_Expand(hTreeWndCtrl, hTreeWnd[i], TVE_EXPAND);
               break;
            case IDM_COLLAPSE:
               TreeView_Expand(hTreeWndCtrl, hTreeCurrent, TVE_COLLAPSE);
               break;
            case IDM_TEST :
               break;
            case IDM_EXIT :
               DestroyWindow(hWnd);
               break;
          }
         break;
    case WM_NOTIFY:
      nmptr = (LPNMHDR) lParam;
      if(nmptr->code == TVN_SELCHANGED) {
        InvalidateRect(hWnd, NULL, 1);
        if(((LPNM_TREEVIEW)nmptr)->itemNew.hItem == hTreeWnd[0])
          strcpy(selection, "One.");
        else if(((LPNM_TREEVIEW)nmptr)->itemNew.hItem == hTreeWnd[1])
          strcpy(selection, "Two.");
        if(((LPNM_TREEVIEW)nmptr)->itemNew.hItem == hTreeWnd[2])
          strcpy(selection, "Three.");
        if(((LPNM_TREEVIEW)nmptr)->itemNew.hItem == hTreeWnd[3])
          strcpy(selection, "Four.");
        if(((LPNM_TREEVIEW)nmptr)->itemNew.hItem == hTreeWnd[4])
          strcpy(selection, "Five.");
       hTreeCurrent = ((LPNM_TREEVIEW)nmptr)->itemNew.hItem;
      }
      break;
      case WM_PAINT:
         hdc = BeginPaint(hWnd, &paintstruct);
         report(hdc, selection);
         EndPaint(hWnd, &paintstruct);
         break;
      case WM_DESTROY :
         PostQuitMessage(0);
         break;
      default:
         return (DefWindowProc(hWnd, uMsg, wParam, lParam));
    }
   return(0L);
 }

/* Initialize the tree list. */
void InitTree(void)
{
  TV_INSERTSTRUCT tvs;
  TV_ITEM tvi;

  tvs.hInsertAfter = TVI_LAST; /* make tree in order given */
  tvi.mask = TVIF_TEXT;

  tvi.pszText = "One";
  tvs.hParent = TVI_ROOT;
  tvs.item = tvi;
  hTreeWnd[0] = TreeView_InsertItem(hTreeWndCtrl, &tvs);
  hTreeCurrent = hTreeWnd[0];

  tvi.pszText = "Two";
  tvs.hParent = hTreeWnd[0];
  tvs.item = tvi;
  hTreeWnd[1] = TreeView_InsertItem(hTreeWndCtrl, &tvs);

  tvi.pszText = "Three";
  tvs.item = tvi;
  tvs.hParent = hTreeWnd[1];
  hTreeWnd[2] = TreeView_InsertItem(hTreeWndCtrl, &tvs);

  tvi.pszText = "Four";
  tvs.item = tvi;
  tvs.hParent = hTreeWnd[2];
  hTreeWnd[3] = TreeView_InsertItem(hTreeWndCtrl, &tvs);

  tvi.pszText = "Five";
  tvs.item = tvi;
  tvs.hParent = hTreeWnd[2];
  hTreeWnd[4] = TreeView_InsertItem(hTreeWndCtrl, &tvs);
}


/* Report Selection */
void report(HDC hdc, char *s)
{
  char str[80];

  if(*s) {
    strcpy(str, "Selection is ");
    strcat(str, s);
  }
  else strcpy(str, "No selection has been made.");
  TextOut(hdc, 0, 200, str, strlen(str));
}


